declare type Entity = {
    [componentName: string]: any;
};
export interface IComponent<T> {
    new (...args: any[]): T;
}
export declare type EntitiesCallback = (entities: Entity[]) => void;
export declare type ChangeCallback<T> = (entity: Entity, updates: ComponentUpdate<T>[]) => void;
export declare type ComponentUpdater<T> = (component: T) => T;
export declare type ComponentUpdate<T> = [IComponent<T>, T | ComponentUpdater<T>];
declare class _Component<T> {
    protected _: T;
    constructor(value: T);
    get(): T;
    find<V>(key: string): V;
    set(value: T): T;
}
export declare type Component<T> = _Component<T> & T;
export declare const Component: new <T>(obj: T) => _Component<T> & T;
export declare class ID extends Component<{
    id: string;
}> {
}
interface Query {
    components: IComponent<unknown>[];
    entities: Entity[];
    subscriptions: EntitiesCallback[];
    changeHandlers: ChangeCallback<any>[];
}
export interface Config {
    parallel?: boolean;
    id?: () => string;
    onBefore?: (...args: any[]) => Promise<void>;
    onAfter?: (...args: any[]) => Promise<void>;
}
export declare function getID(entity: Entity): string;
export declare function getComponent<T>(entity: Entity, Component: IComponent<T>): T;
export declare function hasComponent<T>(entity: Entity, components: IComponent<T>): boolean;
export declare function hasComponents(entity: Entity, components: IComponent<unknown>[]): boolean;
export declare class World {
    protected config: Config;
    protected systems: [Function, string][];
    protected entities: {
        [id: string]: Entity;
    };
    protected queries: {
        [key: string]: Query;
    };
    constructor(config?: Config);
    protected makeQueryKey(components: IComponent<unknown>[]): string;
    protected queryWithKey(key: string, components: IComponent<unknown>[], persist?: Boolean): Entity[];
    private _handleAddCallbacks;
    private _handleRemoveCallbacks;
    addComponent<T>(entity: Entity, Component: IComponent<T>, ...args: any[]): void;
    addComponents(entity: Entity, components: [IComponent<unknown>, ...any[]][]): void;
    createEntity(components: [IComponent<unknown>, ...any[]][]): Entity;
    get(id: string): Entity;
    query(components: IComponent<unknown>[], persist?: Boolean): Entity[];
    register(system: Function, components: IComponent<unknown>[]): void;
    removeComponent<T>(entity: Entity, component: IComponent<T>): void;
    removeComponents(entity: Entity, components: IComponent<unknown>[]): void;
    run(...args: any[]): Promise<void>;
    subscribe(components: IComponent<unknown>[], callback: EntitiesCallback, emit?: boolean): () => void;
    makeSubscription(components: IComponent<unknown>[], emit?: boolean): (cb: EntitiesCallback) => () => void;
    handleChange(components: IComponent<unknown>[], callback: ChangeCallback<unknown>): () => void;
    makeChangeHandler(components: IComponent<unknown>[]): (cb: ChangeCallback<unknown>) => () => void;
    unsubscribe(components: IComponent<unknown>[], callback: EntitiesCallback): void;
    deregisterHandler(components: IComponent<unknown>[], callback: ChangeCallback<unknown>): void;
    updateComponent<T>(entity: Entity, Component: IComponent<T>, update: any | ComponentUpdater<T>): IComponent<T>;
    updateComponents(entity: Entity, updates: ComponentUpdate<any>[]): [IComponent<unknown>, unknown][];
    private _update;
}
export {};
